@@ -121,16 +121,13 @@
    <rsalz@bbn.com> and Jim Berets <jberets@bbn.com> in August, 1990.
 
    Modified by Paul Eggert <eggert@twinsun.com> in August 1999 to do
-   the right thing about local DST, and in February 2004 to support
-   nanosecond-resolution time stamps.  Unlike previous versions, this
-   version is reentrant.  */
+   the right thing about local DST.  Also modified by Paul Eggert
+   <eggert@cs.ucla.edu> in February 2004 to support
+   nanosecond-resolution time stamps, and in October 2004 to support
+   TZ strings in dates.  */
 
 /* FIXME: Check for arithmetic overflow in all cases, not just
-   some of them.
-
-   FIXME: The current code uses 'int' to count seconds; it should use
-   something like 'intmax_t' to support time stamps that don't fit in
-   32 bits.  */
+   some of them.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -152,7 +149,12 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <stdlib.h> /* for `free'; used by Bison 1.27 */
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "setenv.h"
+#include "xalloc.h"
 
 #if STDC_HEADERS || (! defined isascii && ! HAVE_ISASCII)
 # define IN_CTYPE_DOMAIN(c) 1
@@ -163,23 +165,16 @@
 #define ISSPACE(c) (IN_CTYPE_DOMAIN (c) && isspace (c))
 #define ISALPHA(c) (IN_CTYPE_DOMAIN (c) && isalpha (c))
 #define ISLOWER(c) (IN_CTYPE_DOMAIN (c) && islower (c))
-#define ISDIGIT_LOCALE(c) (IN_CTYPE_DOMAIN (c) && isdigit (c))
 
-/* ISDIGIT differs from ISDIGIT_LOCALE, as follows:
+/* ISDIGIT differs from isdigit, as follows:
    - Its arg may be any int or unsigned int; it need not be an unsigned char.
    - It's guaranteed to evaluate its argument exactly once.
    - It's typically faster.
    POSIX says that only '0' through '9' are digits.  Prefer ISDIGIT to
-   ISDIGIT_LOCALE unless it's important to use the locale's definition
+   isdigit unless it's important to use the locale's definition
    of `digit' even when the host does not conform to POSIX.  */
 #define ISDIGIT(c) ((unsigned int) (c) - '0' <= 9)
 
-#include <string.h>
-
-#if USE_UNLOCKED_IO
-# include "unlocked-io.h"
-#endif
-
 #if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 8) || __STRICT_ANSI__
 # define __attribute__(x)
 #endif
@@ -188,6 +183,21 @@
 # define ATTRIBUTE_UNUSED __attribute__ ((__unused__))
 #endif
 
+/* Shift A right by B bits portably, by dividing A by 2**B and
+   truncating towards minus infinity.  A and B should be free of side
+   effects, and B should be in the range 0 <= B <= INT_BITS - 2, where
+   INT_BITS is the number of useful bits in an int.  GNU code can
+   assume that INT_BITS is at least 32.
+
+   ISO C99 says that A >> B is implementation-defined if A < 0.  Some
+   implementations (e.g., UNICOS 9.0 on a Cray Y-MP EL) don't shift
+   right in the usual way when A < 0, so SHR falls back on division if
+   ordinary A >> B doesn't seem to be the usual signed shift.  */
+#define SHR(a, b)	\
+  (-1 >> 1 == -1	\
+   ? (a) >> (b)		\
+   : (a) / (1 << (b)) - ((a) % (1 << (b)) < 0))
+
 #define EPOCH_YEAR 1970
 #define TM_YEAR_BASE 1900
 
@@ -265,12 +275,9 @@ typedef struct
   table local_time_zone_table[3];
 } parser_control;
 
-#define PC (* (parser_control *) parm)
-#define YYLEX_PARAM parm
-#define YYPARSE_PARAM parm
-
-static int yyerror ();
-static int yylex ();
+union YYSTYPE;
+static int yylex (union YYSTYPE *, parser_control *);
+static int yyerror (parser_control *, char *);
 
 
 
@@ -288,14 +295,14 @@ static int yylex ();
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 185 "getdate.y"
+#line 195 "getdate.y"
 typedef union YYSTYPE {
   long int intval;
   textint textintval;
   struct timespec timespec;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 299 "getdate.c"
+#line 306 "getdate.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -307,7 +314,7 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 311 "getdate.c"
+#line 318 "getdate.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -498,13 +505,13 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   205,   205,   206,   210,   217,   219,   223,   225,   227,
-     229,   231,   233,   235,   239,   247,   255,   265,   272,   284,
-     286,   291,   293,   295,   300,   305,   310,   318,   323,   343,
-     350,   358,   366,   371,   377,   382,   391,   401,   405,   407,
-     409,   411,   413,   415,   417,   419,   421,   423,   425,   427,
-     429,   431,   433,   435,   437,   439,   441,   443,   447,   447,
-     450,   451,   456,   457,   462,   500,   501
+       0,   215,   215,   216,   220,   227,   229,   233,   235,   237,
+     239,   241,   243,   245,   249,   257,   265,   275,   282,   294,
+     296,   301,   303,   305,   310,   315,   320,   328,   333,   353,
+     360,   368,   376,   381,   387,   392,   401,   411,   415,   417,
+     419,   421,   423,   425,   427,   429,   431,   433,   435,   437,
+     439,   441,   443,   445,   447,   449,   451,   453,   457,   457,
+     460,   461,   466,   467,   472,   510,   511
 };
 #endif
 
@@ -689,7 +696,7 @@ do								\
     }								\
   else								\
     { 								\
-      yyerror ("syntax error: cannot back up");\
+      yyerror (pc, "syntax error: cannot back up");\
       YYERROR;							\
     }								\
 while (0)
@@ -713,7 +720,7 @@ while (0)
 #ifdef YYLEX_PARAM
 # define YYLEX yylex (&yylval, YYLEX_PARAM)
 #else
-# define YYLEX yylex (&yylval)
+# define YYLEX yylex (&yylval, pc)
 #endif
 
 /* Enable debugging if requested.  */
@@ -967,7 +974,7 @@ int yyparse ();
 # endif
 #else /* ! YYPARSE_PARAM */
 #if defined (__STDC__) || defined (__cplusplus)
-int yyparse (void);
+int yyparse ( parser_control *pc );
 #else
 int yyparse ();
 #endif
@@ -992,11 +999,11 @@ int yyparse (YYPARSE_PARAM)
 #else /* ! YYPARSE_PARAM */
 #if defined (__STDC__) || defined (__cplusplus)
 int
-yyparse (void)
+yyparse ( parser_control *pc )
 #else
 int
-yyparse ()
-
+yyparse (pc)
+     parser_control *pc ;
 #endif
 #endif
 {
@@ -1251,159 +1258,159 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 211 "getdate.y"
+#line 221 "getdate.y"
     {
-	PC.seconds = yyvsp[0].timespec;
-	PC.timespec_seen = true;
+	pc->seconds = yyvsp[0].timespec;
+	pc->timespec_seen = true;
       }
     break;
 
   case 7:
-#line 224 "getdate.y"
-    { PC.times_seen++; }
+#line 234 "getdate.y"
+    { pc->times_seen++; }
     break;
 
   case 8:
-#line 226 "getdate.y"
-    { PC.local_zones_seen++; }
+#line 236 "getdate.y"
+    { pc->local_zones_seen++; }
     break;
 
   case 9:
-#line 228 "getdate.y"
-    { PC.zones_seen++; }
+#line 238 "getdate.y"
+    { pc->zones_seen++; }
     break;
 
   case 10:
-#line 230 "getdate.y"
-    { PC.dates_seen++; }
+#line 240 "getdate.y"
+    { pc->dates_seen++; }
     break;
 
   case 11:
-#line 232 "getdate.y"
-    { PC.days_seen++; }
+#line 242 "getdate.y"
+    { pc->days_seen++; }
     break;
 
   case 12:
-#line 234 "getdate.y"
-    { PC.rels_seen++; }
+#line 244 "getdate.y"
+    { pc->rels_seen++; }
     break;
 
   case 14:
-#line 240 "getdate.y"
+#line 250 "getdate.y"
     {
-	PC.hour = yyvsp[-1].textintval.value;
-	PC.minutes = 0;
-	PC.seconds.tv_sec = 0;
-	PC.seconds.tv_nsec = 0;
-	PC.meridian = yyvsp[0].intval;
+	pc->hour = yyvsp[-1].textintval.value;
+	pc->minutes = 0;
+	pc->seconds.tv_sec = 0;
+	pc->seconds.tv_nsec = 0;
+	pc->meridian = yyvsp[0].intval;
       }
     break;
 
   case 15:
-#line 248 "getdate.y"
+#line 258 "getdate.y"
     {
-	PC.hour = yyvsp[-3].textintval.value;
-	PC.minutes = yyvsp[-1].textintval.value;
-	PC.seconds.tv_sec = 0;
-	PC.seconds.tv_nsec = 0;
-	PC.meridian = yyvsp[0].intval;
+	pc->hour = yyvsp[-3].textintval.value;
+	pc->minutes = yyvsp[-1].textintval.value;
+	pc->seconds.tv_sec = 0;
+	pc->seconds.tv_nsec = 0;
+	pc->meridian = yyvsp[0].intval;
       }
     break;
 
   case 16:
-#line 256 "getdate.y"
+#line 266 "getdate.y"
     {
-	PC.hour = yyvsp[-3].textintval.value;
-	PC.minutes = yyvsp[-1].textintval.value;
-	PC.seconds.tv_sec = 0;
-	PC.seconds.tv_nsec = 0;
-	PC.meridian = MER24;
-	PC.zones_seen++;
-	PC.time_zone = yyvsp[0].textintval.value % 100 + (yyvsp[0].textintval.value / 100) * 60;
+	pc->hour = yyvsp[-3].textintval.value;
+	pc->minutes = yyvsp[-1].textintval.value;
+	pc->seconds.tv_sec = 0;
+	pc->seconds.tv_nsec = 0;
+	pc->meridian = MER24;
+	pc->zones_seen++;
+	pc->time_zone = yyvsp[0].textintval.value % 100 + (yyvsp[0].textintval.value / 100) * 60;
       }
     break;
 
   case 17:
-#line 266 "getdate.y"
+#line 276 "getdate.y"
     {
-	PC.hour = yyvsp[-5].textintval.value;
-	PC.minutes = yyvsp[-3].textintval.value;
-	PC.seconds = yyvsp[-1].timespec;
-	PC.meridian = yyvsp[0].intval;
+	pc->hour = yyvsp[-5].textintval.value;
+	pc->minutes = yyvsp[-3].textintval.value;
+	pc->seconds = yyvsp[-1].timespec;
+	pc->meridian = yyvsp[0].intval;
       }
     break;
 
   case 18:
-#line 273 "getdate.y"
+#line 283 "getdate.y"
     {
-	PC.hour = yyvsp[-5].textintval.value;
-	PC.minutes = yyvsp[-3].textintval.value;
-	PC.seconds = yyvsp[-1].timespec;
-	PC.meridian = MER24;
-	PC.zones_seen++;
-	PC.time_zone = yyvsp[0].textintval.value % 100 + (yyvsp[0].textintval.value / 100) * 60;
+	pc->hour = yyvsp[-5].textintval.value;
+	pc->minutes = yyvsp[-3].textintval.value;
+	pc->seconds = yyvsp[-1].timespec;
+	pc->meridian = MER24;
+	pc->zones_seen++;
+	pc->time_zone = yyvsp[0].textintval.value % 100 + (yyvsp[0].textintval.value / 100) * 60;
       }
     break;
 
   case 19:
-#line 285 "getdate.y"
-    { PC.local_isdst = yyvsp[0].intval; }
+#line 295 "getdate.y"
+    { pc->local_isdst = yyvsp[0].intval; }
     break;
 
   case 20:
-#line 287 "getdate.y"
-    { PC.local_isdst = yyvsp[-1].intval < 0 ? 1 : yyvsp[-1].intval + 1; }
+#line 297 "getdate.y"
+    { pc->local_isdst = yyvsp[-1].intval < 0 ? 1 : yyvsp[-1].intval + 1; }
     break;
 
   case 21:
-#line 292 "getdate.y"
-    { PC.time_zone = yyvsp[0].intval; }
+#line 302 "getdate.y"
+    { pc->time_zone = yyvsp[0].intval; }
     break;
 
   case 22:
-#line 294 "getdate.y"
-    { PC.time_zone = yyvsp[0].intval + 60; }
+#line 304 "getdate.y"
+    { pc->time_zone = yyvsp[0].intval + 60; }
     break;
 
   case 23:
-#line 296 "getdate.y"
-    { PC.time_zone = yyvsp[-1].intval + 60; }
+#line 306 "getdate.y"
+    { pc->time_zone = yyvsp[-1].intval + 60; }
     break;
 
   case 24:
-#line 301 "getdate.y"
+#line 311 "getdate.y"
     {
-	PC.day_ordinal = 1;
-	PC.day_number = yyvsp[0].intval;
+	pc->day_ordinal = 1;
+	pc->day_number = yyvsp[0].intval;
       }
     break;
 
   case 25:
-#line 306 "getdate.y"
+#line 316 "getdate.y"
     {
-	PC.day_ordinal = 1;
-	PC.day_number = yyvsp[-1].intval;
+	pc->day_ordinal = 1;
+	pc->day_number = yyvsp[-1].intval;
       }
     break;
 
   case 26:
-#line 311 "getdate.y"
+#line 321 "getdate.y"
     {
-	PC.day_ordinal = yyvsp[-1].textintval.value;
-	PC.day_number = yyvsp[0].intval;
+	pc->day_ordinal = yyvsp[-1].textintval.value;
+	pc->day_number = yyvsp[0].intval;
       }
     break;
 
   case 27:
-#line 319 "getdate.y"
+#line 329 "getdate.y"
     {
-	PC.month = yyvsp[-2].textintval.value;
-	PC.day = yyvsp[0].textintval.value;
+	pc->month = yyvsp[-2].textintval.value;
+	pc->day = yyvsp[0].textintval.value;
       }
     break;
 
   case 28:
-#line 324 "getdate.y"
+#line 334 "getdate.y"
     {
 	/* Interpret as YYYY/MM/DD if the first value has 4 or more digits,
 	   otherwise as MM/DD/YY.
@@ -1412,252 +1419,252 @@ yyreduce:
 	   you want portability, use the ISO 8601 format.  */
 	if (4 <= yyvsp[-4].textintval.digits)
 	  {
-	    PC.year = yyvsp[-4].textintval;
-	    PC.month = yyvsp[-2].textintval.value;
-	    PC.day = yyvsp[0].textintval.value;
+	    pc->year = yyvsp[-4].textintval;
+	    pc->month = yyvsp[-2].textintval.value;
+	    pc->day = yyvsp[0].textintval.value;
 	  }
 	else
 	  {
-	    PC.month = yyvsp[-4].textintval.value;
-	    PC.day = yyvsp[-2].textintval.value;
-	    PC.year = yyvsp[0].textintval;
+	    pc->month = yyvsp[-4].textintval.value;
+	    pc->day = yyvsp[-2].textintval.value;
+	    pc->year = yyvsp[0].textintval;
 	  }
       }
     break;
 
   case 29:
-#line 344 "getdate.y"
+#line 354 "getdate.y"
     {
 	/* ISO 8601 format.  YYYY-MM-DD.  */
-	PC.year = yyvsp[-2].textintval;
-	PC.month = -yyvsp[-1].textintval.value;
-	PC.day = -yyvsp[0].textintval.value;
+	pc->year = yyvsp[-2].textintval;
+	pc->month = -yyvsp[-1].textintval.value;
+	pc->day = -yyvsp[0].textintval.value;
       }
     break;
 
   case 30:
-#line 351 "getdate.y"
+#line 361 "getdate.y"
     {
 	/* e.g. 17-JUN-1992.  */
-	PC.day = yyvsp[-2].textintval.value;
-	PC.month = yyvsp[-1].intval;
-	PC.year.value = -yyvsp[0].textintval.value;
-	PC.year.digits = yyvsp[0].textintval.digits;
+	pc->day = yyvsp[-2].textintval.value;
+	pc->month = yyvsp[-1].intval;
+	pc->year.value = -yyvsp[0].textintval.value;
+	pc->year.digits = yyvsp[0].textintval.digits;
       }
     break;
 
   case 31:
-#line 359 "getdate.y"
+#line 369 "getdate.y"
     {
 	/* e.g. JUN-17-1992.  */
-	PC.month = yyvsp[-2].intval;
-	PC.day = -yyvsp[-1].textintval.value;
-	PC.year.value = -yyvsp[0].textintval.value;
-	PC.year.digits = yyvsp[0].textintval.digits;
+	pc->month = yyvsp[-2].intval;
+	pc->day = -yyvsp[-1].textintval.value;
+	pc->year.value = -yyvsp[0].textintval.value;
+	pc->year.digits = yyvsp[0].textintval.digits;
       }
     break;
 
   case 32:
-#line 367 "getdate.y"
+#line 377 "getdate.y"
     {
-	PC.month = yyvsp[-1].intval;
-	PC.day = yyvsp[0].textintval.value;
+	pc->month = yyvsp[-1].intval;
+	pc->day = yyvsp[0].textintval.value;
       }
     break;
 
   case 33:
-#line 372 "getdate.y"
+#line 382 "getdate.y"
     {
-	PC.month = yyvsp[-3].intval;
-	PC.day = yyvsp[-2].textintval.value;
-	PC.year = yyvsp[0].textintval;
+	pc->month = yyvsp[-3].intval;
+	pc->day = yyvsp[-2].textintval.value;
+	pc->year = yyvsp[0].textintval;
       }
     break;
 
   case 34:
-#line 378 "getdate.y"
+#line 388 "getdate.y"
     {
-	PC.day = yyvsp[-1].textintval.value;
-	PC.month = yyvsp[0].intval;
+	pc->day = yyvsp[-1].textintval.value;
+	pc->month = yyvsp[0].intval;
       }
     break;
 
   case 35:
-#line 383 "getdate.y"
+#line 393 "getdate.y"
     {
-	PC.day = yyvsp[-2].textintval.value;
-	PC.month = yyvsp[-1].intval;
-	PC.year = yyvsp[0].textintval;
+	pc->day = yyvsp[-2].textintval.value;
+	pc->month = yyvsp[-1].intval;
+	pc->year = yyvsp[0].textintval;
       }
     break;
 
   case 36:
-#line 392 "getdate.y"
+#line 402 "getdate.y"
     {
-	PC.rel_ns = -PC.rel_ns;
-	PC.rel_seconds = -PC.rel_seconds;
-	PC.rel_minutes = -PC.rel_minutes;
-	PC.rel_hour = -PC.rel_hour;
-	PC.rel_day = -PC.rel_day;
-	PC.rel_month = -PC.rel_month;
-	PC.rel_year = -PC.rel_year;
+	pc->rel_ns = -pc->rel_ns;
+	pc->rel_seconds = -pc->rel_seconds;
+	pc->rel_minutes = -pc->rel_minutes;
+	pc->rel_hour = -pc->rel_hour;
+	pc->rel_day = -pc->rel_day;
+	pc->rel_month = -pc->rel_month;
+	pc->rel_year = -pc->rel_year;
       }
     break;
 
   case 38:
-#line 406 "getdate.y"
-    { PC.rel_year += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 416 "getdate.y"
+    { pc->rel_year += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 39:
-#line 408 "getdate.y"
-    { PC.rel_year += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 418 "getdate.y"
+    { pc->rel_year += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 40:
-#line 410 "getdate.y"
-    { PC.rel_year += yyvsp[0].intval; }
+#line 420 "getdate.y"
+    { pc->rel_year += yyvsp[0].intval; }
     break;
 
   case 41:
-#line 412 "getdate.y"
-    { PC.rel_month += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 422 "getdate.y"
+    { pc->rel_month += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 42:
-#line 414 "getdate.y"
-    { PC.rel_month += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 424 "getdate.y"
+    { pc->rel_month += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 43:
-#line 416 "getdate.y"
-    { PC.rel_month += yyvsp[0].intval; }
+#line 426 "getdate.y"
+    { pc->rel_month += yyvsp[0].intval; }
     break;
 
   case 44:
-#line 418 "getdate.y"
-    { PC.rel_day += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 428 "getdate.y"
+    { pc->rel_day += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 45:
-#line 420 "getdate.y"
-    { PC.rel_day += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 430 "getdate.y"
+    { pc->rel_day += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 46:
-#line 422 "getdate.y"
-    { PC.rel_day += yyvsp[0].intval; }
+#line 432 "getdate.y"
+    { pc->rel_day += yyvsp[0].intval; }
     break;
 
   case 47:
-#line 424 "getdate.y"
-    { PC.rel_hour += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 434 "getdate.y"
+    { pc->rel_hour += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 48:
-#line 426 "getdate.y"
-    { PC.rel_hour += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 436 "getdate.y"
+    { pc->rel_hour += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 49:
-#line 428 "getdate.y"
-    { PC.rel_hour += yyvsp[0].intval; }
+#line 438 "getdate.y"
+    { pc->rel_hour += yyvsp[0].intval; }
     break;
 
   case 50:
-#line 430 "getdate.y"
-    { PC.rel_minutes += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 440 "getdate.y"
+    { pc->rel_minutes += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 51:
-#line 432 "getdate.y"
-    { PC.rel_minutes += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 442 "getdate.y"
+    { pc->rel_minutes += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 52:
-#line 434 "getdate.y"
-    { PC.rel_minutes += yyvsp[0].intval; }
+#line 444 "getdate.y"
+    { pc->rel_minutes += yyvsp[0].intval; }
     break;
 
   case 53:
-#line 436 "getdate.y"
-    { PC.rel_seconds += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 446 "getdate.y"
+    { pc->rel_seconds += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 54:
-#line 438 "getdate.y"
-    { PC.rel_seconds += yyvsp[-1].textintval.value * yyvsp[0].intval; }
+#line 448 "getdate.y"
+    { pc->rel_seconds += yyvsp[-1].textintval.value * yyvsp[0].intval; }
     break;
 
   case 55:
-#line 440 "getdate.y"
-    { PC.rel_seconds += yyvsp[-1].timespec.tv_sec * yyvsp[0].intval; PC.rel_ns += yyvsp[-1].timespec.tv_nsec * yyvsp[0].intval; }
+#line 450 "getdate.y"
+    { pc->rel_seconds += yyvsp[-1].timespec.tv_sec * yyvsp[0].intval; pc->rel_ns += yyvsp[-1].timespec.tv_nsec * yyvsp[0].intval; }
     break;
 
   case 56:
-#line 442 "getdate.y"
-    { PC.rel_seconds += yyvsp[-1].timespec.tv_sec * yyvsp[0].intval; PC.rel_ns += yyvsp[-1].timespec.tv_nsec * yyvsp[0].intval; }
+#line 452 "getdate.y"
+    { pc->rel_seconds += yyvsp[-1].timespec.tv_sec * yyvsp[0].intval; pc->rel_ns += yyvsp[-1].timespec.tv_nsec * yyvsp[0].intval; }
     break;
 
   case 57:
-#line 444 "getdate.y"
-    { PC.rel_seconds += yyvsp[0].intval; }
+#line 454 "getdate.y"
+    { pc->rel_seconds += yyvsp[0].intval; }
     break;
 
   case 61:
-#line 452 "getdate.y"
+#line 462 "getdate.y"
     { yyval.timespec.tv_sec = yyvsp[0].textintval.value; yyval.timespec.tv_nsec = 0; }
     break;
 
   case 63:
-#line 458 "getdate.y"
+#line 468 "getdate.y"
     { yyval.timespec.tv_sec = yyvsp[0].textintval.value; yyval.timespec.tv_nsec = 0; }
     break;
 
   case 64:
-#line 463 "getdate.y"
+#line 473 "getdate.y"
     {
-	if (PC.dates_seen
-	    && ! PC.rels_seen && (PC.times_seen || 2 < yyvsp[0].textintval.digits))
-	  PC.year = yyvsp[0].textintval;
+	if (pc->dates_seen
+	    && ! pc->rels_seen && (pc->times_seen || 2 < yyvsp[0].textintval.digits))
+	  pc->year = yyvsp[0].textintval;
 	else
 	  {
 	    if (4 < yyvsp[0].textintval.digits)
 	      {
-		PC.dates_seen++;
-		PC.day = yyvsp[0].textintval.value % 100;
-		PC.month = (yyvsp[0].textintval.value / 100) % 100;
-		PC.year.value = yyvsp[0].textintval.value / 10000;
-		PC.year.digits = yyvsp[0].textintval.digits - 4;
+		pc->dates_seen++;
+		pc->day = yyvsp[0].textintval.value % 100;
+		pc->month = (yyvsp[0].textintval.value / 100) % 100;
+		pc->year.value = yyvsp[0].textintval.value / 10000;
+		pc->year.digits = yyvsp[0].textintval.digits - 4;
 	      }
 	    else
 	      {
-		PC.times_seen++;
+		pc->times_seen++;
 		if (yyvsp[0].textintval.digits <= 2)
 		  {
-		    PC.hour = yyvsp[0].textintval.value;
-		    PC.minutes = 0;
+		    pc->hour = yyvsp[0].textintval.value;
+		    pc->minutes = 0;
 		  }
 		else
 		  {
-		    PC.hour = yyvsp[0].textintval.value / 100;
-		    PC.minutes = yyvsp[0].textintval.value % 100;
+		    pc->hour = yyvsp[0].textintval.value / 100;
+		    pc->minutes = yyvsp[0].textintval.value % 100;
 		  }
-		PC.seconds.tv_sec = 0;
-		PC.seconds.tv_nsec = 0;
-		PC.meridian = MER24;
+		pc->seconds.tv_sec = 0;
+		pc->seconds.tv_nsec = 0;
+		pc->meridian = MER24;
 	      }
 	  }
       }
     break;
 
   case 65:
-#line 500 "getdate.y"
+#line 510 "getdate.y"
     { yyval.intval = MER24; }
     break;
 
   case 66:
-#line 502 "getdate.y"
+#line 512 "getdate.y"
     { yyval.intval = yyvsp[0].intval; }
     break;
 
@@ -1665,7 +1672,7 @@ yyreduce:
     }
 
 /* Line 999 of yacc.c.  */
-#line 1669 "getdate.c"
+#line 1676 "getdate.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1738,15 +1745,15 @@ yyerrlab:
 			yycount++;
 		      }
 		}
-	      yyerror (yymsg);
+	      yyerror (pc, yymsg);
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    yyerror ("syntax error; also virtual memory exhausted");
+	    yyerror (pc, "syntax error; also virtual memory exhausted");
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror ("syntax error");
+	yyerror (pc, "syntax error");
     }
 
 
@@ -1845,7 +1852,7 @@ yyabortlab:
 | yyoverflowlab -- parser overflow comes here.  |
 `----------------------------------------------*/
 yyoverflowlab:
-  yyerror ("parser stack overflow");
+  yyerror (pc, "parser stack overflow");
   yyresult = 2;
   /* Fall through.  */
 #endif
@@ -1859,7 +1866,7 @@ yyreturn:
 }
 
 
-#line 505 "getdate.y"
+#line 515 "getdate.y"
 
 
 static table const meridian_table[] =
@@ -1868,7 +1875,7 @@ static table const meridian_table[] =
   { "A.M.", tMERIDIAN, MERam },
   { "PM",   tMERIDIAN, MERpm },
   { "P.M.", tMERIDIAN, MERpm },
-  { 0, 0, 0 }
+  { NULL, 0, 0 }
 };
 
 static table const dst_table[] =
@@ -1902,7 +1909,7 @@ static table const month_and_day_table[] =
   { "THURS",	tDAY,	 4 },
   { "FRIDAY",	tDAY,	 5 },
   { "SATURDAY",	tDAY,	 6 },
-  { 0, 0, 0 }
+  { NULL, 0, 0 }
 };
 
 static table const time_units_table[] =
@@ -1917,7 +1924,7 @@ static table const time_units_table[] =
   { "MIN",	tMINUTE_UNIT,	 1 },
   { "SECOND",	tSEC_UNIT,	 1 },
   { "SEC",	tSEC_UNIT,	 1 },
-  { 0, 0, 0 }
+  { NULL, 0, 0 }
 };
 
 /* Assorted relative-time words. */
@@ -1943,7 +1950,7 @@ static table const relative_time_table[] =
   { "ELEVENTH",	tUNUMBER,	11 },
   { "TWELFTH",	tUNUMBER,	12 },
   { "AGO",	tAGO,		 1 },
-  { 0, 0, 0 }
+  { NULL, 0, 0 }
 };
 
 /* The time zone table.  This table is necessarily incomplete, as time
@@ -2003,7 +2010,7 @@ static table const time_zone_table[] =
   { "GST",	tZONE,     HOUR (10) },	/* Guam Standard */
   { "NZST",	tZONE,     HOUR (12) },	/* New Zealand Standard */
   { "NZDT",	tDAYZONE,  HOUR (12) },	/* New Zealand Daylight */
-  { 0, 0, 0  }
+  { NULL, 0, 0  }
 };
 
 /* Military time zone table. */
@@ -2034,7 +2041,7 @@ static table const military_table[] =
   { "X", tZONE,	 HOUR (11) },
   { "Y", tZONE,	 HOUR (12) },
   { "Z", tZONE,	 HOUR ( 0) },
-  { 0, 0, 0 }
+  { NULL, 0, 0 }
 };
 
 
@@ -2044,16 +2051,14 @@ to_hour (long int hours, int meridian)
 {
   switch (meridian)
     {
+    default: /* Pacify GCC.  */
     case MER24:
       return 0 <= hours && hours < 24 ? hours : -1;
     case MERam:
       return 0 < hours && hours < 12 ? hours : hours == 12 ? 0 : -1;
     case MERpm:
       return 0 < hours && hours < 12 ? hours + 12 : hours == 12 ? 12 : -1;
-    default:
-      abort ();
     }
-  /* NOTREACHED */
 }
 
 static long int
@@ -2086,7 +2091,7 @@ lookup_zone (parser_control const *pc, char const *name)
     if (strcmp (name, tp->name) == 0)
       return tp;
 
-  return 0;
+  return NULL;
 }
 
 #if ! HAVE_TM_GMTOFF
@@ -2099,12 +2104,12 @@ tm_diff (struct tm const *a, struct tm const *b)
 {
   /* Compute intervening leap days correctly even if year is negative.
      Take care to avoid int overflow in leap day calculations.  */
-  int a4 = (a->tm_year >> 2) + (TM_YEAR_BASE >> 2) - ! (a->tm_year & 3);
-  int b4 = (b->tm_year >> 2) + (TM_YEAR_BASE >> 2) - ! (b->tm_year & 3);
+  int a4 = SHR (a->tm_year, 2) + SHR (TM_YEAR_BASE, 2) - ! (a->tm_year & 3);
+  int b4 = SHR (b->tm_year, 2) + SHR (TM_YEAR_BASE, 2) - ! (b->tm_year & 3);
   int a100 = a4 / 25 - (a4 % 25 < 0);
   int b100 = b4 / 25 - (b4 % 25 < 0);
-  int a400 = a100 >> 2;
-  int b400 = b100 >> 2;
+  int a400 = SHR (a100, 2);
+  int b400 = SHR (b100, 2);
   int intervening_leap_days = (a4 - b4) - (a100 - b100) + (a400 - b400);
   long int ayear = a->tm_year;
   long int years = ayear - b->tm_year;
@@ -2185,7 +2190,7 @@ lookup_word (parser_control const *pc, char *word)
   if (period_found && (tp = lookup_zone (pc, word)))
     return tp;
 
-  return 0;
+  return NULL;
 }
 
 static int
@@ -2351,11 +2356,56 @@ yylex (YYSTYPE *lvalp, parser_control *pc)
 
 /* Do nothing if the parser reports an error.  */
 static int
-yyerror (char *s ATTRIBUTE_UNUSED)
+yyerror (parser_control *pc ATTRIBUTE_UNUSED, char *s ATTRIBUTE_UNUSED)
 {
   return 0;
 }
 
+/* If *TM0 is the old and *TM1 is the new value of a struct tm after
+   passing it to mktime, return true if it's OK that mktime returned T.
+   It's not OK if *TM0 has out-of-range members.  */
+
+static bool
+mktime_ok (struct tm const *tm0, struct tm const *tm1, time_t t)
+{
+  if (t == (time_t) -1)
+    {
+      /* Guard against falsely reporting an error when parsing a time
+	 stamp that happens to equal (time_t) -1, on a host that
+	 supports such a time stamp.  */
+      tm1 = localtime (&t);
+      if (!tm1)
+	return false;
+    }
+
+  return ! ((tm0->tm_sec ^ tm1->tm_sec)
+	    | (tm0->tm_min ^ tm1->tm_min)
+	    | (tm0->tm_hour ^ tm1->tm_hour)
+	    | (tm0->tm_mday ^ tm1->tm_mday)
+	    | (tm0->tm_mon ^ tm1->tm_mon)
+	    | (tm0->tm_year ^ tm1->tm_year));
+}
+
+/* A reasonable upper bound for the size of ordinary TZ strings.
+   Use heap allocation if TZ's length exceeds this.  */
+enum { TZBUFSIZE = 100 };
+
+/* Return a copy of TZ, stored in TZBUF if it fits, and heap-allocated
+   otherwise.  */
+static char *
+get_tz (char tzbuf[TZBUFSIZE])
+{
+  char *tz = getenv ("TZ");
+  if (tz)
+    {
+      size_t tzsize = strlen (tz) + 1;
+      tz = (tzsize <= TZBUFSIZE
+	    ? memcpy (tzbuf, tz, tzsize)
+	    : xmemdup (tz, tzsize));
+    }
+  return tz;
+}
+
 /* Parse a date/time string, storing the resulting time value into *RESULT.
    The string itself is pointed to by P.  Return true if successful.
    P can be an incomplete or relative time specification; if so, use
@@ -2370,6 +2420,11 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
   struct tm tm0;
   parser_control pc;
   struct timespec gettime_buffer;
+  unsigned char c;
+  bool tz_was_altered = false;
+  char *tz0 = NULL;
+  char tz0buf[TZBUFSIZE];
+  bool ok = true;
 
   if (! now)
     {
@@ -2385,6 +2440,44 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
   if (! tmp)
     return false;
 
+  while (c = *p, ISSPACE (c))
+    p++;
+
+  if (strncmp (p, "TZ=\"", 4) == 0)
+    {
+      char const *tzbase = p + 4;
+      size_t tzsize = 1;
+      char const *s;
+
+      for (s = tzbase; *s; s++, tzsize++)
+	if (*s == '\\')
+	  {
+	    s++;
+	    if (! (*s == '\\' || *s == '"'))
+	      break;
+	  }
+	else if (*s == '"')
+	  {
+	    char *z;
+	    char *tz1;
+	    char tz1buf[TZBUFSIZE];
+	    bool large_tz = TZBUFSIZE < tzsize;
+	    bool setenv_ok;
+	    tz0 = get_tz (tz0buf);
+	    z = tz1 = large_tz ? xmalloc (tzsize) : tz1buf;
+	    for (s = tzbase; *s != '"'; s++)
+	      *z++ = *(s += *s == '\\');
+	    *z = '\0';
+	    setenv_ok = setenv ("TZ", tz1, 1) == 0;
+	    if (large_tz)
+	      free (tz1);
+	    if (!setenv_ok)
+	      goto fail;
+	    tz_was_altered = true;
+	    p = s + 1;
+	  }
+    }
+
   pc.input = p;
   pc.year.value = tmp->tm_year;
   pc.year.value += TM_YEAR_BASE;
@@ -2417,7 +2510,7 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
   pc.local_time_zone_table[0].name = tmp->tm_zone;
   pc.local_time_zone_table[0].type = tLOCAL_ZONE;
   pc.local_time_zone_table[0].value = tmp->tm_isdst;
-  pc.local_time_zone_table[1].name = 0;
+  pc.local_time_zone_table[1].name = NULL;
 
   /* Probe the names used in the next three calendar quarters, looking
      for a tm_isdst different from the one we already have.  */
@@ -2434,7 +2527,7 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
 		pc.local_time_zone_table[1].name = probe_tm->tm_zone;
 		pc.local_time_zone_table[1].type = tLOCAL_ZONE;
 		pc.local_time_zone_table[1].value = probe_tm->tm_isdst;
-		pc.local_time_zone_table[2].name = 0;
+		pc.local_time_zone_table[2].name = NULL;
 	      }
 	    break;
 	  }
@@ -2453,10 +2546,10 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
 	pc.local_time_zone_table[i].type = tLOCAL_ZONE;
 	pc.local_time_zone_table[i].value = i;
       }
-    pc.local_time_zone_table[i].name = 0;
+    pc.local_time_zone_table[i].name = NULL;
   }
 #else
-  pc.local_time_zone_table[0].name = 0;
+  pc.local_time_zone_table[0].name = NULL;
 #endif
 #endif
 
@@ -2468,152 +2561,181 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
 	 daylight times.  So if we see that abbreviation, we don't
 	 know whether it's daylight time.  */
       pc.local_time_zone_table[0].value = -1;
-      pc.local_time_zone_table[1].name = 0;
+      pc.local_time_zone_table[1].name = NULL;
     }
 
   if (yyparse (&pc) != 0)
-    return false;
+    goto fail;
 
   if (pc.timespec_seen)
-    {
-      *result = pc.seconds;
-      return true;
-    }
-
-  if (1 < pc.times_seen || 1 < pc.dates_seen || 1 < pc.days_seen
-      || 1 < (pc.local_zones_seen + pc.zones_seen)
-      || (pc.local_zones_seen && 1 < pc.local_isdst))
-    return false;
-
-  tm.tm_year = to_year (pc.year) - TM_YEAR_BASE + pc.rel_year;
-  tm.tm_mon = pc.month - 1 + pc.rel_month;
-  tm.tm_mday = pc.day + pc.rel_day;
-  if (pc.times_seen || (pc.rels_seen && ! pc.dates_seen && ! pc.days_seen))
-    {
-      tm.tm_hour = to_hour (pc.hour, pc.meridian);
-      if (tm.tm_hour < 0)
-	return false;
-      tm.tm_min = pc.minutes;
-      tm.tm_sec = pc.seconds.tv_sec;
-    }
+    *result = pc.seconds;
   else
     {
-      tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
-      pc.seconds.tv_nsec = 0;
-    }
+      if (1 < pc.times_seen || 1 < pc.dates_seen || 1 < pc.days_seen
+	  || 1 < (pc.local_zones_seen + pc.zones_seen)
+	  || (pc.local_zones_seen && 1 < pc.local_isdst))
+	goto fail;
 
-  /* Let mktime deduce tm_isdst if we have an absolute time stamp,
-     or if the relative time stamp mentions days, months, or years.  */
-  if (pc.dates_seen | pc.days_seen | pc.times_seen | pc.rel_day
-      | pc.rel_month | pc.rel_year)
-    tm.tm_isdst = -1;
-
-  /* But if the input explicitly specifies local time with or without
-     DST, give mktime that information.  */
-  if (pc.local_zones_seen)
-    tm.tm_isdst = pc.local_isdst;
-
-  tm0 = tm;
-
-  Start = mktime (&tm);
-
-  if (Start == (time_t) -1)
-    {
-
-      /* Guard against falsely reporting errors near the time_t boundaries
-	 when parsing times in other time zones.  For example, if the min
-	 time_t value is 1970-01-01 00:00:00 UTC and we are 8 hours ahead
-	 of UTC, then the min localtime value is 1970-01-01 08:00:00; if
-	 we apply mktime to 1970-01-01 00:00:00 we will get an error, so
-	 we apply mktime to 1970-01-02 08:00:00 instead and adjust the time
-	 zone by 24 hours to compensate.  This algorithm assumes that
-	 there is no DST transition within a day of the time_t boundaries.  */
-      if (pc.zones_seen)
+      tm.tm_year = to_year (pc.year) - TM_YEAR_BASE;
+      tm.tm_mon = pc.month - 1;
+      tm.tm_mday = pc.day;
+      if (pc.times_seen || (pc.rels_seen && ! pc.dates_seen && ! pc.days_seen))
 	{
-	  tm = tm0;
-	  if (tm.tm_year <= EPOCH_YEAR - TM_YEAR_BASE)
-	    {
-	      tm.tm_mday++;
-	      pc.time_zone += 24 * 60;
-	    }
-	  else
-	    {
-	      tm.tm_mday--;
-	      pc.time_zone -= 24 * 60;
-	    }
-	  Start = mktime (&tm);
+	  tm.tm_hour = to_hour (pc.hour, pc.meridian);
+	  if (tm.tm_hour < 0)
+	    goto fail;
+	  tm.tm_min = pc.minutes;
+	  tm.tm_sec = pc.seconds.tv_sec;
+	}
+      else
+	{
+	  tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
+	  pc.seconds.tv_nsec = 0;
 	}
 
-      if (Start == (time_t) -1)
-	return false;
-    }
+      /* Let mktime deduce tm_isdst if we have an absolute time stamp.  */
+      if (pc.dates_seen | pc.days_seen | pc.times_seen)
+	tm.tm_isdst = -1;
+
+      /* But if the input explicitly specifies local time with or without
+	 DST, give mktime that information.  */
+      if (pc.local_zones_seen)
+	tm.tm_isdst = pc.local_isdst;
+
+      tm0 = tm;
 
-  if (pc.days_seen && ! pc.dates_seen)
-    {
-      tm.tm_mday += ((pc.day_number - tm.tm_wday + 7) % 7
-		     + 7 * (pc.day_ordinal - (0 < pc.day_ordinal)));
-      tm.tm_isdst = -1;
       Start = mktime (&tm);
-      if (Start == (time_t) -1)
-	return false;
-    }
 
-  if (pc.zones_seen)
-    {
-      long int delta = pc.time_zone * 60;
-      time_t t1;
+      if (! mktime_ok (&tm0, &tm, Start))
+	{
+	  if (! pc.zones_seen)
+	    goto fail;
+	  else
+	    {
+	      /* Guard against falsely reporting errors near the time_t
+		 boundaries when parsing times in other time zones.  For
+		 example, suppose the input string "1969-12-31 23:00:00 -0100",
+		 the current time zone is 8 hours ahead of UTC, and the min
+		 time_t value is 1970-01-01 00:00:00 UTC.  Then the min
+		 localtime value is 1970-01-01 08:00:00, and mktime will
+		 therefore fail on 1969-12-31 23:00:00.  To work around the
+		 problem, set the time zone to 1 hour behind UTC temporarily
+		 by setting TZ="XXX1:00" and try mktime again.  */
+
+	      long int time_zone = pc.time_zone;
+	      long int abs_time_zone = time_zone < 0 ? - time_zone : time_zone;
+	      long int abs_time_zone_hour = abs_time_zone / 60;
+	      int abs_time_zone_min = abs_time_zone % 60;
+	      char tz1buf[sizeof "XXX+0:00"
+			  + sizeof pc.time_zone * CHAR_BIT / 3];
+	      if (!tz_was_altered)
+		tz0 = get_tz (tz0buf);
+	      sprintf (tz1buf, "XXX%s%ld:%02d", "-" + (time_zone < 0),
+		       abs_time_zone_hour, abs_time_zone_min);
+	      if (setenv ("TZ", tz1buf, 1) != 0)
+		goto fail;
+	      tz_was_altered = true;
+	      tm = tm0;
+	      Start = mktime (&tm);
+	      if (! mktime_ok (&tm0, &tm, Start))
+		goto fail;
+	    }
+	}
+
+      if (pc.days_seen && ! pc.dates_seen)
+	{
+	  tm.tm_mday += ((pc.day_number - tm.tm_wday + 7) % 7
+			 + 7 * (pc.day_ordinal - (0 < pc.day_ordinal)));
+	  tm.tm_isdst = -1;
+	  Start = mktime (&tm);
+	  if (Start == (time_t) -1)
+	    goto fail;
+	}
+
+      if (pc.zones_seen)
+	{
+	  long int delta = pc.time_zone * 60;
+	  time_t t1;
 #ifdef HAVE_TM_GMTOFF
-      delta -= tm.tm_gmtoff;
+	  delta -= tm.tm_gmtoff;
 #else
-      time_t t = Start;
-      struct tm const *gmt = gmtime (&t);
-      if (! gmt)
-	return false;
-      delta -= tm_diff (&tm, gmt);
+	  time_t t = Start;
+	  struct tm const *gmt = gmtime (&t);
+	  if (! gmt)
+	    goto fail;
+	  delta -= tm_diff (&tm, gmt);
 #endif
-      t1 = Start - delta;
-      if ((Start < t1) != (delta < 0))
-	return false;	/* time_t overflow */
-      Start = t1;
+	  t1 = Start - delta;
+	  if ((Start < t1) != (delta < 0))
+	    goto fail;	/* time_t overflow */
+	  Start = t1;
+	}
+
+      /* Add relative date.  */
+      if (pc.rel_year | pc.rel_month | pc.rel_day)
+	{
+	  int year = tm.tm_year + pc.rel_year;
+	  int month = tm.tm_mon + pc.rel_month;
+	  int day = tm.tm_mday + pc.rel_day;
+	  if (((year < tm.tm_year) ^ (pc.rel_year < 0))
+	      | ((month < tm.tm_mon) ^ (pc.rel_month < 0))
+	      | ((day < tm.tm_mday) ^ (pc.rel_day < 0)))
+	    goto fail;
+	  tm.tm_year = year;
+	  tm.tm_mon = month;
+	  tm.tm_mday = day;
+	  Start = mktime (&tm);
+	  if (Start == (time_t) -1)
+	    goto fail;
+	}
+
+      /* Add relative hours, minutes, and seconds.  On hosts that support
+	 leap seconds, ignore the possibility of leap seconds; e.g.,
+	 "+ 10 minutes" adds 600 seconds, even if one of them is a
+	 leap second.  Typically this is not what the user wants, but it's
+	 too hard to do it the other way, because the time zone indicator
+	 must be applied before relative times, and if mktime is applied
+	 again the time zone will be lost.  */
+      {
+	long int sum_ns = pc.seconds.tv_nsec + pc.rel_ns;
+	long int normalized_ns = (sum_ns % BILLION + BILLION) % BILLION;
+	time_t t0 = Start;
+	long int d1 = 60 * 60 * pc.rel_hour;
+	time_t t1 = t0 + d1;
+	long int d2 = 60 * pc.rel_minutes;
+	time_t t2 = t1 + d2;
+	long int d3 = pc.rel_seconds;
+	time_t t3 = t2 + d3;
+	long int d4 = (sum_ns - normalized_ns) / BILLION;
+	time_t t4 = t3 + d4;
+
+	if ((d1 / (60 * 60) ^ pc.rel_hour)
+	    | (d2 / 60 ^ pc.rel_minutes)
+	    | ((t1 < t0) ^ (d1 < 0))
+	    | ((t2 < t1) ^ (d2 < 0))
+	    | ((t3 < t2) ^ (d3 < 0))
+	    | ((t4 < t3) ^ (d4 < 0)))
+	  goto fail;
+
+	result->tv_sec = t4;
+	result->tv_nsec = normalized_ns;
+      }
     }
 
-  /* Add relative hours, minutes, and seconds.  Ignore leap seconds;
-     i.e. "+ 10 minutes" means 600 seconds, even if one of them is a
-     leap second.  Typically this is not what the user wants, but it's
-     too hard to do it the other way, because the time zone indicator
-     must be applied before relative times, and if mktime is applied
-     again the time zone will be lost.  */
-  {
-    long int sum_ns = pc.seconds.tv_nsec + pc.rel_ns;
-    long int normalized_ns = (sum_ns % BILLION + BILLION) % BILLION;
-    time_t t0 = Start;
-    long int d1 = 60 * 60 * pc.rel_hour;
-    time_t t1 = t0 + d1;
-    long int d2 = 60 * pc.rel_minutes;
-    time_t t2 = t1 + d2;
-    long int d3 = pc.rel_seconds;
-    time_t t3 = t2 + d3;
-    long int d4 = (sum_ns - normalized_ns) / BILLION;
-    time_t t4 = t3 + d4;
+  goto done;
 
-    if ((d1 / (60 * 60) ^ pc.rel_hour)
-	| (d2 / 60 ^ pc.rel_minutes)
-	| ((t1 < t0) ^ (d1 < 0))
-	| ((t2 < t1) ^ (d2 < 0))
-	| ((t3 < t2) ^ (d3 < 0))
-	| ((t4 < t3) ^ (d4 < 0)))
-      return false;
-
-    result->tv_sec = t4;
-    result->tv_nsec = normalized_ns;
-    return true;
-  }
+ fail:
+  ok = false;
+ done:
+  if (tz_was_altered)
+    ok &= (tz0 ? setenv ("TZ", tz0, 1) : unsetenv ("TZ")) == 0;
+  if (tz0 != tz0buf)
+    free (tz0);
+  return ok;
 }
 
 #if TEST
 
-#include <stdio.h>
-
 int
 main (int ac, char **av)
 {
@@ -2622,7 +2744,7 @@ main (int ac, char **av)
   printf ("Enter date, or blank line to exit.\n\t> ");
   fflush (stdout);
 
-  buff[BUFSIZ - 1] = 0;
+  buff[BUFSIZ - 1] = '\0';
   while (fgets (buff, BUFSIZ - 1, stdin) && buff[0])
     {
       struct timespec d;
@@ -2646,6 +2768,6 @@ main (int ac, char **av)
     }
   return 0;
 }
-#endif /* defined TEST */
+#endif /* TEST */
 
 
