@@ -1,5 +1,5 @@
 /* crypto.h --- Crypto prototypes.
- * Copyright (C) 2002, 2003  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -69,8 +69,8 @@ typedef enum
 Shishi_derivekeymode;
 
 typedef int (*Shishi_random_to_key_function) (Shishi * handle,
-					      const char *random,
-					      size_t randomlen,
+					      const char *rnd,
+					      size_t rndlen,
 					      Shishi_key * outkey);
 
 typedef int (*Shishi_string_to_key_function) (Shishi * handle,
@@ -114,7 +114,7 @@ typedef int (*Shishi_verify_function) (Shishi * handle,
 struct cipherinfo
 {
   int32_t type;
-  char *name;
+  const char *name;
   int blocksize;
   int confoundersize;
   int keylen;
@@ -130,7 +130,7 @@ typedef struct cipherinfo cipherinfo;
 struct checksuminfo
 {
   int32_t type;
-  char *name;
+  const char *name;
   int cksumlen;
   Shishi_checksum_function checksum;
   Shishi_verify_function verify;
