@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.18"
+#define SHISHI_VERSION "0.0.19"
 
 /* Error codes */
 typedef enum
@@ -1290,7 +1290,7 @@ extern int shishi_krberror_set_crealm (Shishi * handle,
 				       const char *crealm);
 extern int shishi_krberror_client (Shishi * handle,
 				   Shishi_asn1 krberror,
-				   char **cname, size_t *cnamelen);
+				   char **client, size_t * clientlen);
 extern int shishi_krberror_set_cname (Shishi * handle,
 				      Shishi_asn1 krberror,
 				      Shishi_name_type name_type,
@@ -1964,9 +1964,6 @@ extern Shishi_asn1 shishi_encticketpart (Shishi * handle);
 extern int shishi_encticketpart_key_set (Shishi * handle,
 					 Shishi_asn1 encticketpart,
 					 Shishi_key * key);
-extern int shishi_encticketpart_get_enc_part_etype (Shishi * handle,
-						    Shishi_asn1 encticketpart,
-						    int32_t * etype);
 extern int shishi_encticketpart_get_key (Shishi * handle,
 					 Shishi_asn1 encticketpart,
 					 Shishi_key ** key);
@@ -2110,9 +2107,10 @@ extern const char *shishi_x509key_default_file (Shishi * handle);
 
 /* utils.c */
 extern time_t shishi_get_date (const char *p, const time_t * now);
+extern void shishi_xalloc_die (void);
 
 /* resolv.c */
-Shishi_dns shishi_resolv (const char *zone, uint16_t querytype);
-void shishi_resolv_free (Shishi_dns rrs);
+extern Shishi_dns shishi_resolv (const char *zone, uint16_t querytype);
+extern void shishi_resolv_free (Shishi_dns rrs);
 
 #endif
